
import { ExtensionIdentifier, IViewDescriptor } from '@workbench-stack/core';
import { Event, Emitter } from '@vscode/monaco-editor/esm/vs/base/common/event';
import { Registry } from '@vscode/monaco-editor/esm/vs/platform/registry/common/platform';
import { values, keys } from '@vscode/monaco-editor/esm/vs/base/common/map';
import { localize } from '@vscode/monaco-editor/esm/vs/nls';


export namespace Extensions {
    export const ViewContainersRegistry = 'workbench.registry.view.containers';
    export const ViewsRegistry = 'workbench.registry.view';
}

export interface IViewContainersRegistry {
    /**
     * An event that is triggerred when a view container is registered.
     */
    readonly onDidRegister: Event<IViewContainer>;

    /**
     * An event that is triggerred when a view container is deregistered.
     */
    readonly onDidDeregister: Event<IViewContainer>;

    /**
     * All registered view containers
     */
    readonly all: IViewContainer[];

    /**
     * Registers a view container with given id
     * No op if a view container is already registered with the given id.
     *
     * @param id of the view container.
     *
     * @returns the registered IViewContainer.
     */
    registerViewContainer(id: string, hideIfEmpty?: boolean, extensionId?: ExtensionIdentifier): IViewContainer;

    /**
     * Deregisters the given view container
     * No op if the view container is not registered
     */
    deregisterViewContainer(viewContainer: IViewContainer): void;

    /**
     * Returns the view container with given id.
     *
     * @returns the view container with given id.
     */
    get(id: string): IViewContainer | undefined;
}


export interface IViewContainer {
    id: string;
    hideIfEmpty?: boolean;
    extensionId?: ExtensionIdentifier;
}
class ViewContainersRegistryImpl {


    private viewContainers: Map<string, IViewContainer> = new Map<string, IViewContainer>();

    get all(): IViewContainer[] {
        return values(this.viewContainers);
    }

    public registerViewContainer(id: string, hideIfEmpty?: boolean, extensionId?: ExtensionIdentifier): IViewContainer {
        const existing = this.viewContainers.get(id);
        if (existing) {
            return existing;
        }

        const viewContainer: IViewContainer = {
            id,
            hideIfEmpty,
            extensionId,
        };

        return viewContainer;
    }


    public deregisterViewContainer(viewContainer: IViewContainer): void {
        const existing = this.viewContainers.get(viewContainer.id);
        if (existing) {
            this.viewContainers.delete(viewContainer.id);

        }
    }

    public get(id: string): IViewContainer | undefined {
        return this.viewContainers.get(id);
    }
}

Registry.add(Extensions.ViewContainersRegistry, new ViewContainersRegistryImpl());


export interface IViewsRegistry {
    registerViews(views: IViewDescriptor[], viewContainer: IViewContainer): void;

    deregisterViews(views: IViewDescriptor[], viewContainer: IViewContainer): void;

    moveViews(views: IViewDescriptor[], viewContainer: IViewContainer): void;

    getViews(viewContainer: IViewContainer): IViewDescriptor[];

    getView(id: string): IViewDescriptor | null;

    getViewContainer(id: string): IViewContainer | null;
}

class ViewRegistry implements IViewsRegistry {

    private readonly _onViewsRegistered: Emitter<{ views: IViewDescriptor[], viewContainer: IViewContainer }> = new Emitter<{ views: IViewDescriptor[], viewContainer: IViewContainer }>();
    public readonly onViewsRegistered: Event<{ views: IViewDescriptor[], viewContainer: IViewContainer }> = this._onViewsRegistered.event;

    private readonly _onViewsDeregistered: Emitter<{ views: IViewDescriptor[], viewContainer: IViewContainer }> = new Emitter<{ views: IViewDescriptor[], viewContainer: IViewContainer }>();
    public readonly onViewsDeregistered: Event<{ views: IViewDescriptor[], viewContainer: IViewContainer }> = this._onViewsDeregistered.event;

    // tslint:disable-next-line:max-line-length
    private readonly _onDidChangeContainer: Emitter<{ views: IViewDescriptor[], from: IViewContainer, to: IViewContainer }> = new Emitter<{ views: IViewDescriptor[], from: IViewContainer, to: IViewContainer }>();
    public readonly onDidChangeContainer: Event<{ views: IViewDescriptor[], from: IViewContainer, to: IViewContainer }> = this._onDidChangeContainer.event;

    private _viewContainers: IViewContainer[] = [];
    private _views: Map<IViewContainer, IViewDescriptor[]> = new Map<IViewContainer, IViewDescriptor[]>();


    public registerViews(views: IViewDescriptor[], viewContainer: IViewContainer): void {
        this.addViews(views, viewContainer);
        this._onViewsRegistered.fire({ views: views, viewContainer });
    }

    public deregisterViews(viewDescriptors: IViewDescriptor[], viewContainer: IViewContainer): void {
        const views = this.removeViews(viewDescriptors, viewContainer);
        if (views.length) {
            this._onViewsDeregistered.fire({ views, viewContainer });
        }
    }

    public moveViews(viewsToMove: IViewDescriptor[], viewContainer: IViewContainer): void {
        keys(this._views).forEach(container => {
            if (container !== viewContainer) {
                const views = this.removeViews(viewsToMove, container);
                if (views.length) {
                    this.addViews(views, viewContainer);
                    this._onDidChangeContainer.fire({ views, from: container, to: viewContainer });
                }
            }
        });
    }
    public getViews(loc: IViewContainer): IViewDescriptor[] {
        return this._views.get(loc) || [];
    }

    public getView(id: string): IViewDescriptor | null {
        for (const viewContainer of this._viewContainers) {
            const viewDescriptor = (this._views.get(viewContainer) || []).filter(v => v.id === id)[0];
            if (viewDescriptor) {
                return viewDescriptor;
            }
        }
        return null;
    }

    public getViewContainer(viewId: string): IViewContainer | null {
        for (const viewContainer of this._viewContainers) {
            const viewDescriptor = (this._views.get(viewContainer) || []).filter(v => v.id === viewId)[0];
            if (viewDescriptor) {
                return viewContainer;
            }
        }
        return null;
    }

    private addViews(viewDescriptors: IViewDescriptor[], viewContainer: IViewContainer): void {
        let views = this._views.get(viewContainer);
        if (!views) {
            views = [];
            this._views.set(viewContainer, views);
            this._viewContainers.push(viewContainer);
        }

        for (const viewDescriptor of viewDescriptors) {
            if (views.some(v => v.id === viewDescriptor.id)) {
                throw new Error(localize('duplicateId', "A view with id '{0}' is already registered in the container '{1}'", viewDescriptor.id, viewContainer.id));
            }
            views.push(viewDescriptor);
        }
    }

    private removeViews(viewDescriptors: IViewDescriptor[], viewContainer: IViewContainer): IViewDescriptor[] {
        const views = this._views.get(viewContainer);
        if (!views) {
            return [];
        }
        const viewsToDeregister: IViewDescriptor[] = [];
        const remainingViews: IViewDescriptor[] = [];
        for (const view of views) {
            if (viewDescriptors.indexOf(view) === -1) {
                remainingViews.push(view);
            } else {
                viewsToDeregister.push(view);
            }
        }
        if (viewsToDeregister.length) {
            if (remainingViews.length) {
                this._views.set(viewContainer, remainingViews);
            } else {
                this._views.delete(viewContainer);
                this._viewContainers.splice(this._viewContainers.indexOf(viewContainer), 1);
            }
        }
        return viewsToDeregister;
    }

}


Registry.add(Extensions.ViewsRegistry, new ViewRegistry());

