/* tslint:disable */
import { IJSONSchema } from '@vscode/monaco-editor/esm/vs/base/common/jsonSchema';
import { localize } from '@vscode/monaco-editor/esm/vs/nls';
import { IExtensionPoint } from '@workbench-stack/core';

export interface IUserFriendlyViewsContainerDescriptor {
	id: string;
	title: string;
	icon: string;
}


const viewsContainerSchema: IJSONSchema = {
	type: 'object',
	properties: {
		id: {
			description: localize({ key: 'vscode.extension.contributes.views.containers.id', comment: ['Contribution refers to those that an extension contributes to VS Code through an extension/contribution point. '] }, "Unique id used to identify the container in which views can be contributed using 'views' contribution point"),
			type: 'string',
			pattern: '^[a-zA-Z0-9_-]+$'
		},
		title: {
			description: localize('vscode.extension.contributes.views.containers.title', 'Human readable string used to render the container'),
			type: 'string'
		},
		icon: {
			description: localize('vscode.extension.contributes.views.containers.icon', "Path to the container icon. Icons are 24x24 centered on a 50x40 block and have a fill color of 'rgb(215, 218, 224)' or '#d7dae0'. It is recommended that icons be in SVG, though any image file type is accepted."),
			type: 'string'
		}
	}
};


export const viewsContainersContribution: IJSONSchema = {
    description: localize('vscode.extension.contributes.viewsContainers', 'Contributes views containers to the editor'),
    type: 'object',
    properties: {
        'activitybar': {
            description: localize('views.container.activitybar', 'Contribute views containers to Activity Bar'),
            type: 'array',
            items: viewsContainerSchema
        }
    }
};


const viewDescriptor: IJSONSchema = {
    type: 'object',
    properties: {
        id: {
            description: localize('vscode.extension.contributes.view.id', 'Identifier of the view. Use this to register a data provider through `vscode.window.registerTreeDataProviderForView` API. Also to trigger activating your extension by registering `onView:${id}` event to `activationEvents`.'),
            type: 'string',
        },
        name: {
            description: localize('vscode.extension.contributes.view.name', 'The human-readable name of the view. Will be shown'),
            type: 'string',
        },
        when: {
            description: localize('vscode.extension.contributes.view.when', 'Condition which must be true to show this view'),
            type: 'string',
        },
    }
};


const viewsContribution: IJSONSchema = {
    description: localize('vscode.extension.contributes.views', 'Contributes views to the editor'),
    type: 'object',
    properties: {
        'explorer': {
            description: localize('views.explorer', 'Contributes views to Explorer container in the Activity bar'),
            type: 'array',
            items: viewDescriptor,
            default: [],
        },
        'debug': {
            description: localize('views.debug', 'Contributes views to Debug container in the Activity bar'),
            type: 'array',
            items: viewDescriptor,
            default: [],
        },
        'scm': {
            description: localize('views.scm', 'Contributes views to SCM container in the Activity bar'),
            type: 'array',
            items: viewDescriptor,
            default: [],
        },
        'test': {
            description: localize('views.test', 'Contributes views to Test container in the Activity bar'),
            type: 'array',
            items: viewDescriptor,
            default: [],
        },
    },
    additionalProperties: {
        description: localize('views.contributed', 'Contributes views to contributed views container'),
        type: 'array',
        items: viewDescriptor,
        default: [],
    },
};



type ViewContainerExtensionPointType = { [loc: string]: IUserFriendlyViewsContainerDescriptor[] };
const viewsContainersExtensionPoint: IExtensionPoint<ViewContainerExtensionPointType> = ExtensionsRegistry.registerExtensionPoint<ViewContainerExtensionPointType>({
    extensionPoint: 'viewsContainers',
    jsonSchema: viewsContainersContribution,
});

type ViewExtensionPointType = { [loc: string]: IUserFriendlyViewDescriptor[] };
const viewsExtensionPoint: IExtensionPoint<ViewExtensionPointType> = ExtensionsRegistry.registerExtensionPoint<ViewExtensionPointType>({
    extensionPoint: 'views',
    deps: [viewsContainersExtensionPoint],
    jsonSchema: viewsContribution,
});


class ViewsExtensionHandler implements IWorkbenchContribution {

	private viewContainersRegistry: IViewContainersRegistry;
	private viewsRegistry: IViewsRegistry;

	constructor(
		@IInstantiationService private readonly instantiationService: IInstantiationService
	) {
		this.viewContainersRegistry = Registry.as<IViewContainersRegistry>(ViewContainerExtensions.ViewContainersRegistry);
		this.viewsRegistry = Registry.as<IViewsRegistry>(ViewContainerExtensions.ViewsRegistry);
		this.handleAndRegisterCustomViewContainers();
		this.handleAndRegisterCustomViews();
    }
    

}